
Unit ErrExit;

{ catch run-time errors and print a message }

Interface

Implementation

{$F+}
Var
  ExitSave: pointer;

Const
  MaxArray = 34;   { this unit doesn't handle all error codes }

Type
  ErrorRecordType = record
    ErrNum : Byte;
    ErrText  : String[35];
  End; { Record }

Const
    ErrorArray : array [1..MaxArray] of ErrorRecordType =
     ((ErrNum : 002; ErrText : 'File not found'),
      (ErrNum : 003; ErrText : 'Path not found'),
      (ErrNum : 004; ErrText : 'Too many open files'),
      (ErrNum : 005; ErrText : 'File access denied'),
      (ErrNum : 006; ErrText : 'Invalid file handle'),
      (ErrNum : 012; ErrText : 'Invalid file access code'),
      (ErrNum : 015; ErrText : 'Invalid drive number'),
      (ErrNum : 100; ErrText : 'Disk read error'),
      (ErrNum : 101; ErrText : 'Disk write error'),
      (ErrNum : 102; ErrText : 'File not assigned'),
      (ErrNum : 103; ErrText : 'File not open'),
      (ErrNum : 104; ErrText : 'File not open for input'),
      (ErrNum : 105; ErrText : 'File not open for output'),
      (ErrNum : 106; ErrText : 'Invalid numeric format'),
      (ErrNum : 150; ErrText : 'Disk is write protected'),
      (ErrNum : 152; ErrText : 'Drive not ready'),
      (ErrNum : 153; ErrText : 'Unknown command'),
      (ErrNum : 154; ErrText : 'CRC error in data'),
      (ErrNum : 155; ErrText : 'Bad drive request structure length'),
      (ErrNum : 156; ErrText : 'Disk seek error'),
      (ErrNum : 157; ErrText : 'Unknown media type'),
      (ErrNum : 158; ErrText : 'Sector not found'),
      (ErrNum : 159; ErrText : 'Printer out of paper'),
      (ErrNum : 160; ErrText : 'Device write fault'),
      (ErrNum : 161; ErrText : 'Device read fault'),
      (ErrNum : 162; ErrText : 'Hardware failure'),
      (ErrNum : 200; ErrText : 'Division by zero'),
      (ErrNum : 201; ErrText : 'Range check error'),
      (ErrNum : 202; ErrText : 'Stack overflow error'),
      (ErrNum : 203; ErrText : 'Heap overflow error'),
      (ErrNum : 204; ErrText : 'Invalid pointer operation'),
      (ErrNum : 205; ErrText : 'Floating point overflow'),
      (ErrNum : 206; ErrText : 'Floating point underflow'),
      (ErrNum : 207; ErrText : 'Invalid floating point operation'));


{ output error message, if one, when exiting program }
Procedure FPerrorExit;
Var
  ErrCode,
  I           : Word;
  Msg         : String[80];

  Function Hex(N : LongInt) : String;
  Const
    HexTable : Array[0..15] of Char = '0123456789ABCDEF';
  Begin
    Hex := HexTable[N shr 12]         + HexTable[(N shr 8) and 15] +
           HexTable[(N shr 4) and 15] + HexTable[N and 15]
  End; { Function Hex }

Begin

  { chain to next handler }
  ExitProc := ExitSave;

  { if error (but not User Exit), }
  If (ExitCode > 0) and (ExitCode <> 255) then begin

    { start message with error number }
    ErrCode := ExitCode;
    Str(ErrCode,Msg);

    { find error in table, if possible }
    I := 1;
    While (ErrorArray[I].ErrNum <> ErrCode) and (I <= MaxArray) do
      I := I + 1;

    { only output known error messages }
    If I <= MaxArray then
      Writeln(#13#10#10' Error ',ErrCode,' (',ErrorArray[I].ErrText,
              ') at ',Hex(LongInt(ErrorAddr) shr 16),':',
              Hex(LongInt(ErrorAddr) and $FFFF));

  End; { If }

  { terminate }
  Halt(1)

End; { Procedure FPErrorExit }


Begin

  { save old vector }
  ExitSave := ExitProc;

  { chain to exit handlers }
  ExitProc := @FPerrorExit;

End. { Unit ErrExit }
