
Unit FDecls;

(****************************************************************************
**                                                                         **
**  Constant and variable declarations for low-level things                **
**                                                                         **
****************************************************************************)

Interface

Uses
  Graph,
  Vectors;

Const

  Version                  = '1.20'; { program version                       }
  MajorVersion             =      1;
  MinorVersion             =     20;

  MinGridSize              =   4;    { minimum allowable grid size           }
  MaxGridSize              = 900;    { max allowable grid size               }
  XGridSize      : Word    = 50;     { default grid size                     }
  YGridSize      : Word    = 50;

  ReservedMem    : LongInt = 8096;   { memory reserved from grid             }

  RColor = 0;                        { indices of color components           }
  GColor = 1;
  BColor = 2;

  IgnoreViewPointRange : Boolean = False; { test if viewpoint too close? }

Type

  ParmStr = String[128];

  GridLine = Array[0..MaxGridSize] of Single;

  ColorType = Array[0..2] of Float;

  { how a light source is defined }
  LightRec = Record
    InUse      : Boolean;    { active? }
    Posn       : Cartesian;  { light position }
    Bright     : ColorType;  { light intensity }
    IsSpot     : Boolean;    { is this a spotlight? }
    Direction  : Vector;     { if it's a spot, what direction is it pointing? }
    CosConeAngle,            { cos(cone angle) }
    SpotBWExp  : Float       { and what's the beam width exponent }
  End; { Record }


Const

  FuncStr        : ParmStr = '8*sinc(r)';

  XMinStr        : ParmStr = 'xmin=-5*pi';
  YMinStr        : ParmStr = 'ymin=-5*pi';
  XMaxStr        : ParmStr = 'xmax=5*pi';
  YMaxStr        : ParmStr = 'ymax=5*pi';

  DefaultViewPoint             = 'azelr=20,20,9*pi';
  DefaultFocus                 = 'focus=0,0,0';

  MaxHeight      : Float       = +10000.0; { Z clipping values              }
  MinHeight      : Float       = -10000.0;

  FloatZ         : Boolean = False;  { move Z focus with function value?    }

  FocusSet       : Boolean = False;  { is the focus point set?              }

  MathCheck      : Boolean = False;  { check for math errors?               }
  MathErrValue   : Float   = 0.0;    { replace bad values with this one     }

  FontSize       : Float   =     0;  { font size (start with set flag)      }

  ColorOfText    : Word    = White;  { default text color is white          }

  ShowRange      : Boolean = True;   { show evaluation range?               }
  ShowFunc       : Boolean = True;   { show function value?                 }
  ShowAxes       : Boolean = False;  { show axis labels?                    }
  DrawInfo       : Boolean = False;  { draw above information?              }

  { title at bottom of screen }
  TitleString    : ParmStr = 'Created with FPLOT version ' + Version;

  XAxisLabel     : String[63]  = '- <-- X --> +'; { axis labels             }
  YAxisLabel     : String[63]  = '+ <-- Y --> -';
  ZAxisLabel     : String[63]  = 'Z';

  XAxisTicks     : Integer     =   0; { # ticks along each axis             }
  YAxisTicks     : Integer     =   0;
  ZAxisTicks     : Integer     =   0;

  UserText       : ParmStr = '';     { whatever the user wants to say       }

  UserTextXPos   : Float       =    0.0; { where the user wants to say it   }
  UserTextYPos   : Float       =    0.0;
  UserTextZPos   : Float       =    0.0;

  UserTextXRot   : Float       =    0.0; { and at what angle desired        }
  UserTextYRot   : Float       =    0.0;
  UserTextZRot   : Float       =    0.0;

  Radian         : Boolean = False;  { use radians measure? (no = degrees)  }

  Circular       : Boolean = False;  { circular plot grid?                  }

  TimerOn        : Boolean = True;   { time events?                         }

  Fractal        : Boolean = False;  { do fractal grid?                     }

  Ch             : Char    =    #0;  { keyboard input                       }

  Intensity      : Float   =   1.0;  { lighting intensity sum - system lights }

  XScale         : Float   =   1.0;  { world-space scaling factors           }
  YScale         : Float   =   1.0;
  ZScale         : Float   =   1.0;

  UserScale      : Float   =   1.0;  { uniform screen scaling factor        }
  HScale         : Float   =   1.0;  { screen-space scaling factors         }
  VScale         : Float   =   1.0;

  Discontinuity  : Float   =    30;  { required delta-Z to not draw         }

  WallHeight     : Float   =  64.0;  { bounding box wall height (none yet)  }

  WallBase       : Float   = -32.0;  { base of function bounding box        }

  DelayGiven     : Boolean = False;  { delay command given                  }
  TimeFName      : String[80] = '';  { name of timing output file           }

  PrintImage     : Boolean = False;  { print result of program?             }

  UsePostScript  : Boolean = False;  { print while calculating?             }

  LoadingImage   : Boolean = False;  { did we just load an image file       }
  SavingFile     : Boolean = False;  { are we gonna save a data grid file?  }

  PreCompute     : Boolean = False;  { precompute surface before display?   }

  InverseHalf    : Boolean = False;  { invert halftone colors?              }

  WireFrame      : Boolean = False;  { draw polygon edges?                  }
  Shaded         : Boolean = False;  { draw polygon interiors shaded?       }
  Dithered       : Boolean = False;  { draw polygon interiors dithered      }

  FillColor      : Word    = Black;  { default mesh fill color is black     }
  WireFrameColor : Word    = White;  { default wireframe color is white     }
  BackGndColor   : Word    = Black;  { default background color is black    }

  SurfColor      : ColorType = (1,1,1);  { default spectral surface color   }
  MaxComponent   : Word    = RColor; { default to monochrome light          }

  UseSpec        : Boolean =  True;  { use specular component?              }
  PhongExp       : Integer =   200;  { shininess index - bigger = shinier   }
  SpecMulConst   : Float   =   0.4;  { scaling factor for specular result   }
  DiffMulConst   : Float   =   1.0;  { scaling factor for diffuse result    }

  { lighting information }
  MaxLights = 8;
  Lights : Array[1..MaxLights] of LightRec = (
    (InUse:True;  Posn:(x:0.0; y:0.0; z:100.0); Bright:(8500,0.0,0.0)),
    (InUse:False; Posn:(x:0.0; y:0.0; z:  0.0); Bright:(-1.0,0.0,0.0)),
    (InUse:False; Posn:(x:0.0; y:0.0; z:  0.0); Bright:(-1.0,0.0,0.0)),
    (InUse:False; Posn:(x:0.0; y:0.0; z:  0.0); Bright:(-1.0,0.0,0.0)),
    (InUse:False; Posn:(x:0.0; y:0.0; z:  0.0); Bright:(-1.0,0.0,0.0)),
    (InUse:False; Posn:(x:0.0; y:0.0; z:  0.0); Bright:(-1.0,0.0,0.0)),
    (InUse:False; Posn:(x:0.0; y:0.0; z:  0.0); Bright:(-1.0,0.0,0.0)),
    (InUse:False; Posn:(x:0.0; y:0.0; z:  0.0); Bright:(-1.0,0.0,0.0)));

  
  AmbientLight : ColorType = (0, 0, 0);       { ambient lighting intensity   }

  Translucency : ColorType = (0.5, 0.5, 0.5); { surface translucency         }

  ColorBase = 16;                    { base color for drawing                }

  DoDisa         : Boolean = False;  { disassemble code?                     }

  { "animation" control information }
  AnimName       : String  =    '';  { name of animation view file           }
  AnimSave       : Boolean = False;  { save a new screen view when done?     }
  AnimLoad       : Boolean = False;  { load a screen view sequence?          }
  AnimDelay      : Word    =     0;  { time to delay between frames          }
  AnimFrame      : Integer =    -1;  { frame to replace during save          }

  Blur           : Boolean = False;  { blur the image slightly before saving }

  DoAnimate      : Boolean = False;  { perform an animation?                 }
  AnimStart      : Float   =     0;  { animation start parameter             }
  AnimStop       : Float   =     1;  { animation stop parameter              }
  AnimFrames     : Integer =     1;  { # frames in animation                 }
  AnimParm       : Float   =     0;  { current animation parameter           }
  AnimReverse    : Boolean = False;  { reverse direction at animation end?   }

  { calculation frames }
  MaxCalcFrames  : Integer =     1;  { # frames to calc }
  CalcFrame      : Integer =     1;  { current frame being calculated }

  { PostScript information }
  ScreenNum      : Integer =    11;  { PostScript screen to use              }
  ScreenAngle    : Float   =  45.0;  { PostScript screen angle               }
  ScreenFreq     : Float   =  50.0;  { PostScript screen frequency           }
  PSLineWidth    : Float   =   0.0;  { PostScript line width                 }
  NCopies        : Integer =     1;  { PostScript - copies of page printed   }
  NumScreens = 17;
  PSScreens : Array[1..NumScreens] of string[160] = (
    {  1 Dot           } 'dup mul exch dup mul add 1 exch sub',
    {  2 Dot*          } 'abs exch abs 2 copy add 1 gt {1 sub dup mul exch 1 ' +
                         'sub dup mul add 1 sub} {dup mul exch dup mul add 1 ' +
                         'exch sub} ifelse',
    {  3 Inverted Dot  } 'dup mul exch dup mul add 1 sub',
    {  4 Ring          } 'dup mul exch dup mul add 0.6 exch sub abs -0.5 mul',
    {  5 Inverted Ring } 'dup mul exch dup mul add 0.6 exch sub abs 0.5 mul',
    {  6 Triangle      } 'add 2 div',
    {  7 Triangle*     } '2 exch sub exch abs 2 mul sub 3 div',
    {  8 Grid          } '2 copy abs exch abs gt {exch} if pop 2 mul 1 exch ' +
                         'sub 3.5 div',
    {  9 Diamond       } 'abs exch abs add 1 exch sub',
    { 10 Line          } 'pop',
    { 11 Microwaves    } '/wy exch def 180 mul cos 2 div wy dup dup dup mul ' +
                         'mul sub mul wy add 180 mul cos',
    { 12 Ellipse       } 'dup 5 mul 8 div mul exch dup mul exch add sqrt 1 ' +
                         'exch sub',
    { 13 RoundBox      } 'dup mul dup mul exch dup mul dup mul add 1 exch sub',
    { 14 Custom        } 'dup mul exch dup mul add sqrt 1 exch sub',
    { 15 Star          } 'abs exch abs 2 copy gt {exch} if 1 sub dup 0 eq ' +
                         '{0.01 add} if atan 360 div',
    { 16 Random        } 'pop pop rand 1 add 10240 mod 5120 div 1 exch sub',
    { 17 Line          } 'pop abs 2 mul 1 exch sub'
  );

  NAbortPrefix = '*&*';   { prefix to pass to Abort to prevent FatalError
                            message appearing before the rest of the text }

  ComputeTime  : LongInt = 0; { time to compute surface }
  DisplayTime  : LongInt = 0; { time to display surface }
  PrintTime    : LongInt = 0; { time to print screen }

  DoPlasma          : Boolean = False;  { plasma surface base? }
  PosPlasmaExponent : Float = 0.0;      { plasma post + exponent (0 if no) }
  NegPlasmaExponent : Float = 0.0;      { plasma post - exponent (0 if no) }
  PlasmaSmooth      : Float = 0.0;      { plasma smoothing radius (0 if no) }

Var

  DelayMilli,                 { delay (in milliseconds) until exit          }
  DelayCount    : LongInt;    { count into delay                            }

  XMin,YMin,                  { evaluation grid minimum                     }
  XMax,YMax     : Float;      { evaluation grid maximum                     }

  Grid : Array[0..MaxGridSize] of ^GridLine; { evaluation grid data         }

  PXMax, PYMax  : Integer;    { screen max coordinates                      }

  ViewPoint     : Cartesian;  { eye location                                }
  Focus         : Cartesian;  { focus location                              }

  ViewPolar     : Boolean;    { polar coords for viewpoint?                 }
  FocusPolar    : Boolean;    { polar coords for focus?                     }

  ViewParse     : Boolean;    { need to parse viewpoint info?               }
  FocusParse    : Boolean;    { need to parse focus point info?             }

  ViewText      : Array[1..3] of ParmStr; { viewpoint text components       }
  FocusText     : Array[1..3] of ParmStr; { focus text components           }

  Halftones     : Array[0..64] of Array[0..7] of Byte; { halftone patterns  }

Procedure SignOff(ShowTiming : Boolean);
Procedure Abort(Msg : String);
Function  Int2Str(N : LongInt) : String;
Function  Float2Str(F : Float; N : Integer) : String;
Function  GetKey : Char;
Procedure PrintXY(X,Y : Integer; DataStr : String);
Function  atan2(Y,X : Float) : Float;
Function  Angle(A : Float) : Float;
Procedure PolarToRect(Var Parm1, Parm2, Parm3 : Float);

Implementation

Uses
  Timer,
  CRT;

{---------------------------------------------------------------------------}

Procedure Abort(Msg : String);
{
  Give error message and dump out of program
}
Begin

  { at this point, it doesn't matter if it's not initialized }
  CloseGraph;

  { if not really an error message, just dump out to  }
  If Copy(Msg, 1, Length(NAbortPrefix)) <> NAbortPrefix then
    Writeln(#13#10#10'FatalError: ', Msg)
  Else If Copy(Msg, Length(NAbortPrefix) + 1, 255) <> '' then
    Writeln(#13#10#10, Copy(Msg, Length(NAbortPrefix), 255))
  Else
    SignOff(False);

  { dump out to DOS }
  Halt(1)

End; { Procedure Abort }

{---------------------------------------------------------------------------}

Procedure SignOff(ShowTiming : Boolean);
{
  Display statistics before exiting.
}
Var
  F : Text;
Begin

  DirectVideo := False;

  Writeln('This program is permanently in a state of arrested development');
  Writeln('It is provided so that others may learn the effects of bad');
  Writeln('initial design on the future of a project.');
  Writeln;
  Writeln('Author: Joe Slayton -- jslayton@ridgecrest.ca.us');
  Writeln;
  Writeln('The documentation is in Word for Windows 6.0/95 format');
  Writeln('If you''re using this program, good luck!');
  Writeln;


  { output timing information, if user desires and not using image file }
  If ShowTiming then begin
    Writeln;
    Writeln('For a ',XGridSize+1,'x',YGridSize+1,' point surface,');
    Write('Compute: ':15);
    If PreCompute then
      Writeln(Ticks2Secs(Word(ComputeTime)):0:2,' seconds')
    Else
      Writeln('N/A');
    Writeln('Display: ':15,Ticks2Secs(Word(DisplayTime)):0:2,' seconds');
    Write('Print: ':15);
    If PrintImage then
      Writeln(Ticks2Secs(Word(PrintTime)):0:2,' seconds')
    Else
      Writeln('N/A');
  End; { If }

  If TimeFName <> '' then begin
    Assign(F, TimeFName);
    {$I-}
    Append(F);
    Writeln(F, XGridSize, #9, YGridSize, #9, DisplayTime);
    Close(F);
    {$I+}
  End; { If }

End; { Procedure SignOff }

{---------------------------------------------------------------------------}

Function Int2Str(N : LongInt) : String;
{
  Convert integer to string
}
Var
  S : String;
Begin
  Str(N,S);
  Int2Str := S
End; { Function Int2Str }

{---------------------------------------------------------------------------}

Function Float2Str(F : Float; N : Integer) : String;
{
  Convert real to string
}
Var
  S : String;
Begin
  Str(F:0:N,S);
  Float2Str := S
End; { Function Float2Str }

{---------------------------------------------------------------------------}

Function GetKey : Char;
{
  Get a possibly extended key code
}
Var
  Ch : Char;
Begin
  Ch := ReadKey;
  If Ch = #0 then
    Ch := Chr($80 or Ord(ReadKey));
  GetKey := Ch
End; { Function GetKey }

{---------------------------------------------------------------------------}

Procedure PrintXY(X,Y : Integer; DataStr : String);
{
  Draw text on black background blocks
}
Var
  S : String;
Begin

  { clear space }
  SetColor(BackGndColor);
  S[0] := DataStr[0];
  FillChar(S[1],Length(DataStr),#219);
  OutTextXY(X,Y,S);

  { write data }
  SetColor(ColorOfText);
  OutTextXY(X,Y,DataStr)

End; { Procedure PrintXY }

{---------------------------------------------------------------------------}

Function atan2(Y,X : Float) : Float;
{
  Return the angle of a right triangle, given the lengths of the legs.
  Adjusted for quadrant.
}
Var
  z,w : Float;
Begin

  If X = 0 then
    if Y < 0 then
      z := -pi / 2
    else if y = 0 then
      z := 0.0
    else
      z := pi / 2
  Else if Y = 0 then
    If X < 0 then
      z := pi
    Else
      z := 0
  Else begin

    w := 0.0;

    If (y < 0.0) and (x < 0.0) then
      w := -PI
    Else if (x < 0.0) then
      w := PI;

    z := w + arctan(Y / X);

(*
    While z < 0 do
      z := z + 2 * pi;
    While z > 2 * pi do
      z := z - 2 * pi
*)

  End; { Else }

  atan2 := z

End; { Function atan2 }

{---------------------------------------------------------------------------}

Function Angle(A : Float) : Float;
{
  Return angle in current coordinate system (degree or radian).
  The input angle is in degrees.
}
Begin
  If not Radian then
    A := A * Pi / 180.0;
  Angle := A
End; { Function Angle }

{---------------------------------------------------------------------------}

Procedure PolarToRect(Var Parm1, Parm2, Parm3 : Float);
{ convert polar coordinate to rectangular coordinate.
  On entry:
    Parm1 = azimuth angle
    Parm2 = elevation angle
    Parm3 = range
  On exit:
    Parm1 = X coordinate
    Parm2 = Y coordinate
    Parm3 = Z coordinate
}
Var
  Theta, Phi, Range : Float;
Begin

  { convert to correct angle system }
  Theta := Angle(Parm1);
  Phi   := Angle(Parm2);

  { and make sure that the range is non-negative }
  Range := abs(Parm3);

  { convert to rectangular coords }
  Parm1 := cos(Theta) * cos(Phi) * Range;
  Parm2 := sin(Theta) * cos(Phi) * Range;
  Parm3 :=              sin(Phi) * Range

End; { Procedure PolarToRect }

{---------------------------------------------------------------------------}

End. { Unit FDecls }
