
Unit SetGraph;

(****************************************************************************
**                                                                         **
**  Set graphics mode                                                      **
**                                                                         **
****************************************************************************)

Interface

Const

  ColorMasks : Array[0..15] of Array[0..2] of Single = (
    { R     G     B                      }
    (0.00, 0.00, 0.00 ),  { BLACK        }
    (0.00, 0.00, 0.75 ),  { BLUE         }
    (0.00, 0.75, 0.00 ),  { GREEN        }
    (0.00, 0.00, 0.00 ),  { CYAN         }
    (0.75, 0.00, 0.00 ),  { RED          }
    (0.75, 0.00, 0.75 ),  { MAGENTA      }
    (0.67, 0.67, 0.00 ),  { BROWN        }
    (0.75, 0.75, 0.75 ),  { LIGHTGRAY    }
    (0.50, 0.50, 0.50 ),  { DARKGRAY     }
    (0.50, 0.50, 1.00 ),  { LIGHTBLUE    }
    (0.50, 1.00, 0.50 ),  { LIGHTGREEN   }
    (0.50, 1.00, 1.00 ),  { LIGHTCYAN    }
    (1.00, 0.50, 0.50 ),  { LIGHTRED     }
    (1.00, 0.50, 1.00 ),  { LIGHTMAGENTA }
    (1.00, 1.00, 0.00 ),  { YELLOW       }
    (1.00, 1.00, 1.00 )); { WHITE        }


Procedure GraphicsOn;

Implementation

Uses
  CRT,
  Vectors,
  Graph,
  Print,
  View,
  GDrivers,
  FDecls;

{-------------------------------------------------------------------------}

Procedure SetMRGBPalette(ColorStart,HowMany : Integer; Var Data);
{
  Set a block of DAC registers
}
Begin
  InLine($B8/$12/$10/        { MOV    AX,1012                              }
         $8B/$9E/ColorStart/ { MOV    BX,[BP+ColorStart]                   }
         $8B/$8E/HowMany/    { MOV    CX,[BP+HowMany]                      }
         $C4/$96/Data/       { LES    DX,[BP+Data]                         }
         $CD/$10)            { INT    10                                   }
End; { Procedure SetMRGBPalette }

{$F-}

{-------------------------------------------------------------------------}

Procedure GraphicsOn;
{
  Set up hardware in graphics mode.
}

Var
  XAsp,YAsp         : Word;    { used to query aspect ratio }
  I                 : Integer; { loop counter }
  AutoDetectPointer : Pointer; { scratch for data stuff }
  Scale             : Float;   { rough image stretch factor }
  Palette           : Array[0..63] of Record R,G,B : Byte End; { palette data }
  S                 : String;
  GD, GM            : Integer; { detected video device and mode }

Begin

  DetectGraph(GD, GM);

  { force snow control on CGA machines }
  CheckSnow := GD = CGA;

  { if we can and need to do shaded drawing and no mode, set one }
  If Shaded and (GD in [VGA, MCGA]) and (GraphMode = Detect) then
    GraphMode := GM322008  { 320x200 8-bit color mode }
  { if we're trying to be shaded and wrong device, dither instead }
  Else If Shaded and not (GD in [VGA, MCGA]) then
    Dithered := True;

  { force monochrome display for non-EGA and non-VGA systems }
  If not (GD in [EGA, VGA]) then begin
    WireFrameColor := 1;
    FillColor      := Black;
    ColorOfText    := 1;
    BackGndColor   := Black;
    MaxComponent   := RColor
  End; { If }

  { turn on the grpahics driver (driver handles all errors) }
  InitGDriver;

  { set screen background user requested }
  If BackGndColor <> Black then begin
    SetFillStyle(1,BackGndColor);
    Bar(0,0,GetMaxX,GetMaxY)
  End; { If }

  { make sure Writes, etc. go through BIOS to screen }
  DirectVideo := False;

  { get system aspect ratio }
  GetAspectRatio(XAsp,YAsp);

  { determine max screen size }
  PXMax := GetMaxX;
  PYMax := GetMaxY;

  { take 90% of the smaller screen coord as the scaling distance }
  If PXMax > PYMax then
    Scale := ((PYMax * 0.90) / (XMax - XMin)) * DistBetween(Viewpoint,Focus)
  Else
    Scale := ((PXMax * 0.90) / (YMax - YMin)) * DistBetween(Viewpoint,Focus);

  { manufacture scales for horizontal and vertical directions }
  SetScalingFactors(HScale * UserScale * Scale * (YAsp / XAsp),
                    VScale * UserScale * Scale);

  { init transform matrix }
  GetViewMat(ViewPoint,Focus);

  { init intensity scale }
  If Shaded then begin

    { make sure that the fill color isn't black }
    If FillColor = Black then
      FillColor := White;

    For I := 0 to 63 do
      With Palette[I] do begin
        R := Round(I * ColorMasks[FillColor][0]);
        G := Round(I * ColorMasks[FillColor][1]);
        B := Round(I * ColorMasks[FillColor][2])
      End; { With }

    SetMRGBPalette(ColorBase, 64, Palette)

  End; { Else }

  { set up PostScript printer if required }
  If UsePostScript then begin

    { open output to the printer }
    OpenPrinter;

    { print stock postscript header }
    PrintStr('%! PS-Adobe-2.0'#13#10);
    PrintStr('%%Title FPLOT Output'#13#10);
    PrintStr('%%EndComments'#13#10);

    { output how many copies }
    If NCopies > 1 then begin
      PrintStr('%%BeginSetup'#13#10);
      PrintStr('/#copies ' + Int2Str(NCopies) + ' def'#13#10);
      PrintStr('%%EndSetup'#13#10);
    End; { If }

    { print sideways in center of page }
    PrintStr('300 400 translate 90 rotate gsave');

    { set line end caps }
    PrintStr(#13#10'1 setlinecap 1 setlinejoin ');

    { set the appropriate line width }
    PrintStr(Float2Str(PSLineWidth,2) + ' setlinewidth');

    { scale to counteract screen aspect ratio correction }
    PrintStr(#13#10 +
             Float2Str(((600 / PYMax) * XAsp / YAsp) / 100,4) + ' ' +
             Float2Str(((600 / PYMax)              ) / 100,4) + ' scale'#13#10);

    { determine whether or not color component in command }
    If not Dithered and not Shaded then
      S := '1 setgray '
    Else
      S := '100 div setgray ';

    { draw polygon with or without edges, depending }
    PrintStr('/p { newpath moveto lineto lineto lineto closepath '#13#10);
    If WireFrame then begin
      PrintStr('     gsave ' + S + 'fill grestore'#13#10);
      PrintStr('     0 setgray stroke'#13#10);
    End { If }
    Else
      PrintStr('     ' + S + 'fill'#13#10);
    PrintStr('   } bind def'#13#10);

    { set up to draw line }
    PrintStr('/l { 0 setgray newpath moveto lineto stroke } bind def'#13#10);

    { set a new halftone spot function }
    If Shaded then
      PrintStr(Int2Str(Trunc(ScreenFreq)) + ' ' +
               Int2Str(Trunc(ScreenAngle)) + ' ' +
               ' {' + PSScreens[ScreenNum] + '} setscreen'#13#10);

  End { If }

End; { Procedure GraphicsOn }

{-------------------------------------------------------------------------}

End.
